/*
 * Masina.c
 *
 * Created: 17.05.2019 16:40:13
 * Author : Cristian Prodan
 */ 

#define F_CPU 16000000
#define QUARTER 0xFF/4
#define HALF 0xFF/2
#define THREE_QUARTERS 3*0xFF/4
#define MAX_POWER 0xFF

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "usart.h"

char received_data = 'x';
char blinking_red_lights = 0;
char blinking_left_lights = 0;
char blinking_right_lights = 0;
char blinking_back_right_lights = 0;
char blinking_back_left_lights = 0;
char global_idle_timer;
char speaker_back = 0;

void timer0_init() {
	TCCR0A |= (1 << WGM01) | (1 << WGM00) | (1 << COM0A1);
	OCR0A = 0x00;
	TCCR0B |= (1 << CS00);
}

void timer2_init() {
	TCCR2A |= (1 << WGM21) | (1 << WGM20) | (1 << COM2A1);
	OCR2A = 0x00;
	TCCR2B |= (1 << CS20);
}

void activate_front_leds()
{
	/* Turn on all leds */
	PORTA |= (1 << PA0);
	PORTA |= (1 << PA1);
}

void deactivate_front_leds()
{
	/* Turn on all leds */
	PORTA &= ~(1 << PA0);
	PORTA &= ~(1 << PA1);
}

void back_leds()
{
	blinking_red_lights %= 8;
	speaker_back %= 6;
	
	if(blinking_red_lights >= 4)
		PORTC |= (1 << PC1);
	if(blinking_red_lights < 4)
		PORTC &= ~(1 << PC1);
		
	if(speaker_back == 3)
		PORTA |= (1 << PA2);
	if(speaker_back == 0)
		PORTA &= ~(1 << PA2);
	
	speaker_back++;
	blinking_red_lights++;
}

void leds_right()
{
	blinking_right_lights %= 8;
	
	if(blinking_right_lights >= 4) {
		PORTA |= (1 << PA1);
		PORTC |= (1 << PC0);
		
	}
	
	if(blinking_right_lights < 4) {
		PORTA &= ~(1 << PA1);
		PORTC &= ~(1 << PC0);
	}
	
	blinking_right_lights++;
}


void leds_left()
{
	blinking_left_lights %= 8;
	
	if(blinking_left_lights >= 4) {
		PORTA |= (1 << PA0);
		PORTC |= (1 << PC2);
		
	}
	
	if(blinking_left_lights < 4) {
		PORTA &= ~(1 << PA0);
		PORTC &= ~(1 << PC2);
	}
	
	blinking_left_lights++;
}

void back_right()
{
	PORTC |= (1 << PC1);
	blinking_back_right_lights %= 8;
	
	if(blinking_back_right_lights >= 4) {
		PORTC |= (1 << PC0);
		
	}
	
	if(blinking_back_right_lights < 4) {
		PORTC &= ~(1 << PC0);
	}
	
	blinking_back_right_lights++;
}

void back_left()
{
	PORTC |= (1 << PC1);
	blinking_back_left_lights %= 8;
	
	if(blinking_back_left_lights >= 4) {
		PORTC |= (1 << PC2);
		
	}
	
	if(blinking_back_left_lights < 4) {
		PORTC &= ~(1 << PC2);
	}
	
	blinking_back_left_lights++;
}

void activate_rotate_leds()
{
	PORTA |= (1 << PA0);
	PORTA |= (1 << PA1);
	PORTC |= (1 << PC0);
	PORTC |= (1 << PC2);
}

void deactivate_rotate_leds() 
{
	PORTA &= ~(1 << PA0);
	PORTA &= ~(1 << PA1);
	PORTC &= ~(1 << PC0);
	PORTC &= ~(1 << PC2);
}

void reset_leds()
{
	/* Deactivate all leds */
	PORTD &= ~(1 << PD7);
	PORTA &= ~(1 << PA0);
	PORTA &= ~(1 << PA1);
	PORTA &= ~(1 << PA2);
	PORTC &= ~(1 << PC0);
	PORTC &= ~(1 << PC1);
	PORTC &= ~(1 << PC2);
}

void reset_engine()
{
	PORTD &= ~(1 << PD7);
	PORTD &= ~(1 << PD6);
	PORTB &= ~(1 << PB4);
	PORTB &= ~(1 << PB3);
	
	OCR2A = 0x00;
	OCR0A = 0x00;
}

void set_engine_forward()
{
	PORTD |= (1 << PD7);
	PORTD &= ~(1 << PD6);
	PORTB |= (1 << PB4);
	PORTB &= ~(1 << PB3);
}

void set_engine_backward()
{
	PORTD &= ~(1 << PD7);
	PORTD |= (1 << PD6);
	PORTB &= ~(1 << PB4);
	PORTB |= (1 << PB3);
}

ISR(USART0_RX_vect)
{	
	PORTA &= ~(1 << PA7);
	PORTA &= ~(1 << PA5);
	PORTA |= (1 << PA6);
	
	global_idle_timer = 0;
	received_data = USART0_receive();
	switch (received_data)
			{
				/* 100% forward */
				case 'x':	set_engine_forward();
							OCR2A = MAX_POWER;
							OCR0A = 0x00;
							activate_front_leds();
				break;
				
				/* 75% forward */
				case 'y':	set_engine_forward();
							OCR2A = THREE_QUARTERS;
							OCR0A = QUARTER;
							activate_front_leds();
				break;
				
				/* 50% forward */
				case 'z':	set_engine_forward();
							OCR2A = HALF;
							OCR0A = HALF;
							activate_front_leds();
				break;
				
				/* 25% forward */
				case 'w':	set_engine_forward();
							OCR2A = QUARTER;
							OCR0A = THREE_QUARTERS;
							activate_front_leds();
				break;
				
				/* 100% backwards */
				case 'a':	set_engine_backward();
							OCR2A = 0x00;
							OCR0A = MAX_POWER;
							back_leds();
				break;
				
				/* 50% backwards */
				case 'b':	set_engine_backward();
							OCR2A = HALF;
							OCR0A = HALF;
							back_leds();
						
				break;
				
				
				/* Makes controls janky so leave them out */
				
				/* Left */
				case 'l':	/*PORTD |= (1 << PD7);
							PORTD &= ~(1 << PD6);
							PORTB &= ~(1 << PB4);
							PORTB |= (1 << PB3);
							OCR2A = MAX_POWER;
							OCR0A = MAX_POWER;*/
				break;
				
				/* Right */
				case 'r':	/*PORTD &= ~(1 << PD7);
							PORTD |= (1 << PD6);
							PORTB |= (1 << PB4);
							PORTB &= ~(1 << PB3);
							OCR2A = 0x00;
							OCR0A = 0x00;*/
				break;
				
				/* 100% forward - right */
				case 'c':	set_engine_forward();
							OCR2A = 0x00;
							OCR0A = 0x00;
							leds_right();
				break;
				
				/* 75% forward - right */
				case 'd':	set_engine_forward();
							OCR2A = 0x00;
							OCR0A = QUARTER;
							leds_right();
				break;
				
				/* 50% forward - right */
				case 'e':	set_engine_forward();
							OCR2A = 0x00;
							OCR0A = HALF;
							leds_right();
				break;
				
				/* 25% forward - right */
				case 'f':	set_engine_forward();
							OCR2A = 0x00;
							OCR0A = THREE_QUARTERS;
							leds_right();
				break;
				
				/* 100% forward - left */
				case 'g':	set_engine_forward();
							OCR2A = MAX_POWER;
							OCR0A = MAX_POWER;
							leds_left();
				break;
				
				/* 75% forward - left */
				case 'h':	set_engine_forward();
							OCR2A = THREE_QUARTERS;
							OCR0A = MAX_POWER;
							leds_left();
				break;
				
				/* 50% forward - left */
				case 'i':	set_engine_forward();
							OCR2A = HALF;
							OCR0A = MAX_POWER;
							leds_left();
				break;
				
				/* 25% forward - left */
				case 'j':	set_engine_forward();
							OCR2A = QUARTER;
							OCR0A = MAX_POWER;
							leds_left();
				break;
				
				/* 100% backward - right */
				case 'k':	set_engine_backward();
							OCR2A = MAX_POWER;
							OCR0A = MAX_POWER;
							back_right();
				break;
				
				/* 50% backward - right */
				case 'o':	set_engine_backward();
							OCR2A = HALF;
							OCR0A = MAX_POWER;
							back_right();
				break;
				
				/* 100% backward - left */
				case 'm':	set_engine_backward();
							OCR2A = 0x00;
							OCR0A = 0x00;
							back_left();
				break;
				
				/* 50% backward - left */
				case 'n':	set_engine_backward();
							OCR2A = 0x00;
							OCR0A = HALF;
							back_left();
				break;
				
				default:	/* Should never get here */
				break;
			}
			
			_delay_ms(100);
			
			reset_leds();
			reset_engine();
}

void idle_program() {
	int i;
	
	if(global_idle_timer == 0) {
		reset_engine();
		return;
	}
	
	for (i = 0; i < 10; ++i) {
		if(global_idle_timer == 0)
			break;
		PORTD |= (1 << PD7);
		PORTD &= ~(1 << PD6);
		PORTB |= (1 << PB4);
		PORTB &= ~(1 << PB3);
		OCR2A = HALF;
		OCR0A = HALF;
		activate_front_leds();
		_delay_ms(100);
	}
	
	reset_engine();
	
	deactivate_front_leds();
	
	if(global_idle_timer == 0) {
		reset_engine();
		return;
	}
		
	
	for (i = 0; i < 10; ++i) {
		if(global_idle_timer == 0)
			break;
		PORTD &= ~(1 << PD7);
		PORTD |= (1 << PD6);
		PORTB &= ~(1 << PB4);
		PORTB |= (1 << PB3);
		OCR2A = HALF;
		OCR0A = HALF;
		back_leds();
		_delay_ms(100);
	}
	
	blinking_red_lights = 0;
	
	reset_engine();
	
	if(global_idle_timer == 0)
		return;
	
	for (i = 0; i < 20; ++i) {
		if(global_idle_timer == 0)
			break;
		PORTA |= (1 << PA2);
		_delay_ms(250);
		PORTA &= ~(1 << PA2);
		_delay_ms(250);
	}
	
	reset_engine();
	
}

void second_idle_program()
{
	int i;
	
	if(global_idle_timer == 0) {
		reset_engine();
		return;
	}
	
	activate_rotate_leds();
	
	for (i = 0; i < 15; ++i) {
		if(global_idle_timer == 0)
			break;
		PORTD |= (1 << PD7);
		PORTD &= ~(1 << PD6);
		PORTB &= ~(1 << PB4);
		PORTB |= (1 << PB3);
		OCR2A = MAX_POWER;
		OCR0A = MAX_POWER;
		_delay_ms(100);
	}
	
	reset_engine();
	
	deactivate_rotate_leds();
	_delay_ms(3000);
	activate_rotate_leds();
	
	if(global_idle_timer == 0) {
		deactivate_rotate_leds();
		reset_engine();
		return;
	}
	
	for (i = 0; i < 17; ++i) {
		if(global_idle_timer == 0)
			break;
		PORTD &= ~(1 << PD7);
		PORTD |= (1 << PD6);
		PORTB |= (1 << PB4);
		PORTB &= ~(1 << PB3);
		OCR2A = 0x00;
		OCR0A = 0x00;
		_delay_ms(100);
	}
	
	deactivate_rotate_leds();
	
	reset_engine();
		
	if(global_idle_timer == 0)
		return;
	
	for (i = 0; i < 20; ++i) {
		if(global_idle_timer == 0)
			break;
		PORTA |= (1 << PA2);
		_delay_ms(250);
		PORTA &= ~(1 << PA2);
		_delay_ms(250);
	}
	
	reset_engine();
}

void third_idle_program()
{
	int i;
	
	if(global_idle_timer == 0)
		return;
	
	for(i = 0; i < 10; ++i) {				
		PORTA |= (1 << PA2);
		_delay_ms(500);
		
		if(global_idle_timer == 0)
			break;
		
		PORTA &= ~(1 << PA2);
		_delay_ms(500);
		
		if(global_idle_timer == 0)
			break;
		
		PORTA |= (1 << PA2);
		_delay_ms(500);
		
		if(global_idle_timer == 0)
			break;
		
		PORTA &= ~(1 << PA2);
		_delay_ms(500);
		
		if(global_idle_timer == 0)
			break;
		
		PORTA |= (1 << PA2);
		_delay_ms(500);
		
		if(global_idle_timer == 0)
			break;
		
		PORTA &= ~(1 << PA2);
		_delay_ms(500);
		
		if(global_idle_timer == 0)
			break;
		
		PORTA |= (1 << PA2);
		_delay_ms(500);
		
		if(global_idle_timer == 0)
			break;
		
		PORTA &= ~(1 << PA2);
		_delay_ms(500);
		
		if(global_idle_timer == 0)
			break;
		
		PORTA |= (1 << PA2);
		_delay_ms(400);
		
		if(global_idle_timer == 0)
			break;
		
		PORTA &= ~(1 << PA2);
		_delay_ms(350);
		
		if(global_idle_timer == 0)
			break;
		
		PORTA |= (1 << PA2);
		_delay_ms(50);
		
		if(global_idle_timer == 0)
			break;
		
		PORTA &= ~(1 << PA2);
		_delay_ms(50);
		
		if(global_idle_timer == 0)
			break;
		
		PORTA |= (1 << PA2);
		_delay_ms(50);
		
		if(global_idle_timer == 0)
			break;
		
		PORTA &= ~(1 << PA2);
		_delay_ms(50);
		
		if(global_idle_timer == 0)
			break;
		
	}
}

int main(void)
{
	/* Set PB2 as entry point */
	DDRB &= ~(1 << PB2);
	
	/* Activate pull-up resistance */
	PORTB |= (1 << PB2);
	
	/* Set outputs */
	DDRD |= (1 << PD7);
	
	/* Front leds */
	DDRA |= (1 << PA0);
	DDRA |= (1 << PA1);
	
	DDRD |= (1 << PD6);
	DDRB |= (1 << PB4);
	DDRB |= (1 << PB3);
	
	/* Buzzer */
	DDRA |= (1 << PA2);
	
	/* Back leds */
	DDRC |= (1 << PC0);
	DDRC |= (1 << PC1);
	DDRC |= (1 << PC2);
	
	/* Multi led */
	DDRA |= (1 << PA5);
	DDRA |= (1 << PA6);
	DDRA |= (1 << PA7);
	
	/* Turning off leds */
	PORTD &= ~(1 << PD7);
	PORTA &= ~(1 << PA0);
	PORTA &= ~(1 << PA1);
	PORTA &= ~(1 << PA2);
	PORTA |= (1 << PA5);
	PORTA &= ~(1 << PA6);
	PORTA &= ~(1 << PA7);
	PORTC &= ~(1 << PC0);
	PORTC &= ~(1 << PC1);
	PORTC &= ~(1 << PC2);
	
	
	/* Initialize USART communication */
	USART0_init();
	
	/* Initialize timers */
	timer0_init();
	timer2_init();
	int idle_timer = 0;
	sei();
	
	_delay_ms(2000);

    while (1) 
    {	
		
		PORTA |= (1 << PA7);
		PORTA &= ~(1 << PA5);
		PORTA &= ~(1 << PA6);
		
		reset_engine();
		
		if (global_idle_timer <= 100) {
			idle_timer %= 10;
			if(idle_timer == 0) {
				PORTA |= (1 << PA0);
				PORTA |= (1 << PA1);
				PORTC |= (1 << PC0);
				PORTC |= (1 << PC2);
			}
			
			_delay_ms(100);
			if (idle_timer == 5) {
				PORTA &= ~(1 << PA0);
				PORTA &= ~(1 << PA1);
				PORTC &= ~(1 << PC0);
				PORTC &= ~(1 << PC2);
			}
			
			_delay_ms(100);
			idle_timer++;
			global_idle_timer++;
		} else {
			PORTA |= (1 << PA7);
			PORTA &= ~(1 << PA5);
			PORTA |= (1 << PA6);
			
			reset_leds();		
			idle_program();
			
			_delay_ms(3000);
			
			reset_leds();
			second_idle_program();
			
			_delay_ms(3000);
			
			reset_leds();
			third_idle_program();
		}	
    }
}

